from datetime import datetime
from typing import List

from dateutil.relativedelta import relativedelta


GRANULARITY_STEP = {
    "seconds": lambda x: relativedelta(seconds=x),
    "minutes": lambda x: relativedelta(minutes=x),
    "hours": lambda x: relativedelta(hours=x),
    "days": lambda x: relativedelta(days=x),
    "weeks": lambda x: relativedelta(weeks=x),
    "months": lambda x: relativedelta(months=x),
    "years": lambda x: relativedelta(years=x),
}

START_MESSAGE = """
    ------------------------------------
    Runing dobt iteration {i} / {iter_no}
    ------------------------------------
    Start interval:    {start}
    End interaval:     {end}
    Increment:         {increment}
    Granularity:       {granularity}
    ------------------------------------
    Current timestamp: {start_timestamp}
    Next timestamp:    {next_timestamp}
    ------------------------------------
"""

DOBT_TIME_ARGS = [
    {
        "name_or_flag": ["--start", "-s"],
        "default": datetime.now().strftime("%Y-%m-%d"),
        "help": "Start timestamp for dobt looping.",
    },
    {
        "name_or_flag": ["--end", "-e"],
        "default": datetime.now().strftime("%Y-%m-%d"),
        "help": "End timestamp for dobt looping.",
    },
    {
        "name_or_flag": ["--increment"],
        "default": 1,
        "type": int,
        "nargs": "?",
        "help": "Granularity of time interval.",
    },
    {
        "name_or_flag": ["--granularity"],
        "default": "days",
        "nargs": "?",
        "choices": ["seconds", "minutes", "hours", "days", "weeks", "months", "years"],
        "help": "Granularity of time interval.",
    },
]

DOBT_MAT_ARGS = [
    {
        "name_or_flag": ["-mat", "--mater-type"],
        "nargs": "+",
        "choices": ["table", "view", "incremental"],
        "help": "Filter models by materialization type.",
    }
]

DOBT_GIT_ARGS = [
    {
        "name_or_flag": ["-cc", "--cached-changes"],
        "action": "store_true",
        "help": "Run only cached(staged) and untracked models.",
    },
    {
        "name_or_flag": ["-cd", "--commit-diff"],
        "action": "store_true",
        "help": """
            Run all models changed accourding to diff from provided commit id to HEAD.
        """,
    },
    {
        "name_or_flag": ["-ci", "--commit-id"],
        "default": "HEAD^",
        "help": "Id of commit to be compared to HEAD. Default HEAD^.",
    },
]

DOBT_COMPARE_ARGS = [
    {
        "name_or_flag": ["--database"],
        "nargs": "+",
        "help": "Select only specific database.",
    },
    {
        "name_or_flag": ["--schema"],
        "nargs": "+",
        "help": "Select only specific schema.",
    },
]

DOBT_DELETE_ARGS = [
    {
        "name_or_flag": ["--delete-all"],
        "action": "store_true",
        "help": "Delete all stale relations without confirm prompt. Be careful!",
    },
]

GIT_CURRENT_CHANGES = ["git", "ls-files", "--exclude-standard", "-ocm"]


def get_commit_diff(commit_id: str) -> List[str]:
    return ["git", "diff", "--name-only", "--diff-filter=ACMR", commit_id, "HEAD"]
