import attr
from dbt.logger import GLOBAL_LOGGER as logger
from dobt.task.base import DOBTDBSchemaTask
from typing import Optional, Tuple, List, Any
from jinja2 import Template
from pathlib import Path

SOURCE_YML_TEMPLATE=Template(
    """
version: 2

sources:
  - name: {{schema}}
    database: {{database}}
    schema: {{schema}}
    tables:
    {% for relation in relations -%}
      - name: {{relation['name']}}
        meta:
          - relation_type: {{relation['name']}}
        columns:
        {% for col in relation['columns'] -%}
          - name: {{col['name']}}
        {% endfor %}
    {% endfor %}
    """
)

@attr.s(auto_attribs=True)
class GenerateTask(DOBTDBSchemaTask):
    path: Optional[str] = attr.ib(default=None)
    single_file: boolean = attr.ib(default=True)
    
    def _render_relations(self, relations):
        return SOURCE_YML_TEMPLATE.render(relations=relations)
    
    def _write_relation(self):
        if not self.path:
            self.path = self.config.source_paths[0]
    
    def _write_relations(self, db_rels):
        if single_file:
            for db_rel in db_rels:
                rendered = self._render_relations(relations)
        

            
    
    def _get_relations(self):
        logger.info("Getting list of relations...")
        result = {}
        for db, schema in self.db_schemas:
            rels = self.adapter.list_relations(db, schema)
            result.setdefault((db, schema), [])
            for rel in rels:
                cols = self.adapter.get_columns_in_relation(rel)
                relation = {
                    "database": rel.database,
                    "schema": rel.schema,
                    "name": rel.name,
                    "type": str(rel.type),
                    "columns": [{"name": col.name, "type": col.data_type} for col in cols]
                }
                result[(db, schema)].append(relation)
        if result:
            return result
        else:
            raise RuntimeException(
                f"No relations found in selected schemas."
            )
            
            
    def run(self):
        self._get_adapter()
        with self.adapter.connection_named("dobt_generate"):
            self._set_db_schema_combination()
            relations = self._get_relations()
            self._write_relations(relations)

if __name__ == "__main__":
    task = GenerateTask()
    task.run()