from typing import Any, Dict, List, Optional, Set, Tuple

from dbt.exceptions import RuntimeException
from dbt.logger import GLOBAL_LOGGER as logger
from dbt.ui.printer import green, red

import attr
from dobt.task.compare import CompareTask


@attr.s(auto_attribs=True)
class DeleteTask(CompareTask):
    delete_all: bool = attr.ib(default=False)

    @classmethod
    def from_args(cls, args):
        return cls(
            database=args.database,
            schema=args.schema,
            delete_all=args.delete_all,
            args=args,
        )

    def _delete(self, db_obj):
        self.adapter.drop_relation(db_obj)
        logger.info(red(f"{db_obj.database}.{db_obj.schema}.{db_obj.name} - deleted!"))

    @staticmethod
    def _get_input(db_obj):
        confirm = input(
            f"You are about to drop the {db_obj.type}"
            f" `{db_obj.database}.{db_obj.schema}.{db_obj.name}`. Are you sure [Y/n]? "
        )
        return confirm

    def _confirmation(self, db_obj):
        confirm = self._get_input(db_obj)
        if confirm.lower() in ["y", "yes"]:
            self._delete(db_obj)
        elif confirm.lower() in ["n", "no"]:
            pass
        else:
            self._confirmation(db_obj)  # pragma: no covered

    def run(self):
        db_only, _ = self.compare()
        if db_only:
            with self.adapter.connection_named("dobt_delete"):
                for rel in db_only:
                    db_obj = self.rels_db.get(rel)
                    if self.delete_all:
                        self._delete(db_obj)
                    else:
                        self._confirmation(db_obj)
            logger.info(green("That is all!"))
        else:
            logger.info(green("There are no stale relations in your database!"))
        return [], True
